%option 8bit reentrant
%option warn
%option yylineno
%option noyywrap
%option prefix="conf"
%option extra-type="struct lexical_extra*"
%option bison-bridge
%option bison-locations
%option nounput
%option stack
%x PSTRING
%x PARSE
%x INCLUDE
%x PREPROC

%{
#include "common.h"
#include "parser.h"
#include "object.h"
// flex use register keyword, but it deprecated in C++11.
#define register
// Flex always think what bison types prefixed as YY, so we mast define right macroses
#define YYSTYPE CONFSTYPE
#define YYLTYPE CONFLTYPE
#include "grammatical.h"
// Get rid of warning on unused function
#define YY_NO_INPUT
// struct yyguts_t defined in each flex output file, this emits warning at linking stage
#define yyguts_t confguts_t
// Termination (mostly, in case of errors)
#define yyterminate(ret) { while(0!=YY_CURRENT_BUFFER) yypop_buffer_state(yyscanner); str.erase(); yyextra->retcode=-(ret); return (ret); }
// Abnormal termination with description of error
#define yyerrormessage(message,ret) {\
COUT(ERROR)<<std::endl<<" Lexical error in file "<<yyextra->state.curdir<<yyextra->state.filename<<" at line "<<yyextra->state.curline<<": "<<(message)<<std::endl;\
for(const auto &i:yylloc_param->incstack) COUT(ERROR)<<"   included from "<<i.filename<<" at line "<<i.line<<", position "<<i.column<<std::endl;\
yyterminate(ret);}
// Trace position
#define yynextline {yyextra->state.curline++; yyextra->state.curpos=0; yyextra->state.curoffset++;}
#define yynextsym {yyextra->state.curpos+=yyleng; yyextra->state.curoffset+=yyleng;}
// and set bison location accordingly
#define setyyllocp {yylloc_param->first_line=yylloc_param->last_line=yyextra->state.curline; yylloc_param->first_column=yyextra->state.curpos; yylloc_param->last_column=yylloc_param->first_column+yyleng;}

std::string str;
int incpos,incline;
%}

%%
<PARSE,INITIAL>@[Ii][Nn][Cc][Ll][Uu][Dd][Ee][ \t]*\" yy_push_state(INCLUDE,yyscanner); str.erase(); incpos=yyextra->state.curpos; incline=yyextra->state.curline; yynextsym;
@                                               BEGIN(PREPROC); yynextsym;
<PREPROC>[a-zA-Z]*                              {
                                                 yynextsym; str=yytext; tolower(str);
                                                 if("use"==str) return DIR_USE;
                                                 if("includepath"==str) return DIR_INCLUDEPATH;
                                                 if("modulepath"==str) return DIR_MODULEPATH;
                                                 if("as"==str) return DIR_AS;
                                                 COUT(WARNING)<<std::endl<<"Unknown directive "<<str<<", ignoring"<<std::endl;
                                                }
<INCLUDE>\"                                     {
                                                 if(yyextra->state.inclevel>=yyextra->maxinclevel) yyerrormessage("maximal include level reached",-1);
                                                 FILE* fd;
                                                 std::string fullname;

                                                 // yyextra->curdir is directory with currently scanned file
                                                 // !!!NONPORTABLE!!!
                                                 fullname=str;
                                                 if('/'==fullname[0]) fd=fopen(fullname.c_str(),"r");     // absolute path
                                                 else
                                                 {
                                                  // first, try to search in directory with currently scanned file
                                                  fullname=yyextra->state.curdir+str;
                                                  fd=fopen(fullname.c_str(),"r");
                                                  // if fail, try to search file in includedirs
                                                  if(0==fd)
                                                  {
                                                   std::string curpath;
                                                   size_t bpos=0,epos;
                                                   do
                                                   {
                                                    epos=yyextra->includedirs.find(':',bpos);
                                                    curpath=yyextra->includedirs.substr(bpos,(std::string::npos==epos)?epos:(epos-bpos));
                                                    fullname=curpath+str;
                                                    fd=fopen(fullname.c_str(),"r");
                                                    if(0!=fd) break;
                                                    bpos=epos+1;
                                                   } while(std::string::npos!=epos);
                                                  }
                                                 }

                                                 if(0==fd) yyerrormessage("can't open file "+str,-1);
                                                 yylloc_param->incstack.push_front({incline,incpos,yylloc_param->filename});
                                                 yynextsym;
                                                 yyextra->fds.push(fd);
                                                 yyextra->states.push(yyextra->state);
                                                 COUT(DEBUG)<<std::endl<<"Include "<<str<<std::endl;

                                                 yyextra->state.inclevel++;
                                                 yyextra->ParsePath(fullname);
                                                 yyextra->state.curline=1;
                                                 yyextra->state.curpos=yyextra->state.curoffset=0;
                                                 yylloc_param->filename=yyextra->state.curdir+yyextra->state.filename;

                                                 yypush_buffer_state(yy_create_buffer(fd,YY_BUF_SIZE,yyscanner),yyscanner);
                                                 // Get rid of warning on unused function
                                                 if(__builtin_expect(yy_top_state(yyscanner)==PSTRING,0)) yyerrormessage("misterious error",-1);
                                                 yy_pop_state(yyscanner);
                                                }
[a-zA-Z][a-zA-Z0-9_]*                           COUT(MOREDEBUG)<<"NAME("<<yytext<<")"; BEGIN(PARSE); yylval_param->str=new std::string(yytext); setyyllocp; yynextsym; return NAME;
<PARSE>[+\-*/^]                                 COUT(MOREDEBUG)<<" OPERATION("<<yytext<<")"; setyyllocp; yynextsym; return yytext[0];
<PARSE>[0-9]+   				{
						 COUT(MOREDEBUG)<<" INTEGER("<<yytext<<")";
						 if(!str2int(yytext,&yylval_param->i)) yyerrormessage(std::string("failed conversion to integer of ")+yytext+" at position "+ToString(yyextra->state.curpos),-1);
						 yylval_param->i=atoll(yytext);
						 setyyllocp;
						 yynextsym;
						 return INTEGER;
						}
<PARSE>[0-9]+(\.[0-9]*)?([eE][+-][0-9]+)?	{
						 COUT(MOREDEBUG)<<" REAL("<<yytext<<")";
						 if(!str2double(yytext,&yylval_param->r)) yyerrormessage(std::string("failed conversion to double of ")+yytext+" at position "+ToString(yyextra->state.curpos),-1);
						 setyyllocp;
						 yynextsym;
						 return REAL;
						}
<PARSE>[TF]                                     COUT(MOREDEBUG)<<" BOOL("<<yytext<<")"; yylval_param->b=(yytext[0]=='T')?true:false; setyyllocp; yynextsym; return BOOL;
<PARSE>\(					COUT(MOREDEBUG)<<" OBRACE()"; setyyllocp; yynextsym; return OBRACE;
<PARSE>\)					COUT(MOREDEBUG)<<" CBRACE()"; setyyllocp; yynextsym; return CBRACE;
<PARSE>\;					COUT(MOREDEBUG)<<" ENDL()"<<std::endl; BEGIN(0); setyyllocp; yynextsym; return ENDL;
<PARSE>=					COUT(MOREDEBUG)<<" ASSIGN()"; setyyllocp; yynextsym; return ASSIGN;
<PARSE>,                                        COUT(MOREDEBUG)<<" DELIM()"; setyyllocp; yynextsym; return DELIM;
<PARSE>[a-zA-Z][a-zA-Z0-9_]*		        COUT(MOREDEBUG)<<" IDENTIFIER("<<yytext<<")"; yylval_param->str=new std::string(yytext); setyyllocp; yynextsym; return IDENTIFIER;
<PARSE>\.                                       COUT(MOREDEBUG)<<" DOT()"; setyyllocp; yynextsym; return yytext[0];
<PARSE,PREPROC,INITIAL>[ \t]+                   yynextsym;
<PARSE,INITIAL>\n                               yynextline;
<PREPROC>\n                                     BEGIN(0); yynextline;
<PARSE,INITIAL>\#.*                             yynextsym;
<PARSE,PREPROC>\"			        yy_push_state(PSTRING,yyscanner); str.erase(); yyextra->states.push(yyextra->state); yynextsym;
<PARSE,PREPROC,INITIAL>.                        yyerrormessage(std::string("unknown symbol ")+yytext+" at position "+ToString(yyextra->state.curpos),-1);
<PSTRING,INCLUDE>\\\\				str+='\\'; yynextsym;
<PSTRING,INCLUDE>\\\"	        		str+='\"'; yynextsym;
<PSTRING>\n                                     str+=yytext[0]; yynextline;
<PSTRING>\"					yy_pop_state(yyscanner); COUT(MOREDEBUG)<<" STRING("<<str<<")"; yylval_param->str=new std::string(str); {struct lexical_state prstate=yyextra->states.top(); yyextra->states.pop(); yylloc_param->first_line=prstate.curline; yylloc_param->last_line=yyextra->state.curline; yylloc_param->first_column=prstate.curpos; yylloc_param->last_column=yyextra->state.curpos+yyleng;}; yynextsym; return STRING;
<PSTRING,INCLUDE>.				str+=yytext[0]; yynextsym;
<PSTRING,INCLUDE><<EOF>>			yyextra->states.pop(); yyerrormessage("unclosed quote",-1);
<<EOF>>						yypop_buffer_state(yyscanner); if(0!=YY_CURRENT_BUFFER) {yyextra->state=yyextra->states.top(); yyextra->states.pop(); fclose(yyextra->fds.top()); yyextra->fds.pop(); yylloc_param->filename=yyextra->state.curdir+yyextra->state.filename; yylloc_param->incstack.pop_front();} else yyterminate(0);
%%
